@@ -39,6 +39,7 @@
 #include "kmixtoolbox.h"
 #include "mixdevicewidget.h"
 #include "mixer.h"
+#include "mixertoolbox.h"
 
 
 ViewBase::ViewBase(QWidget* parent, const char* id, Mixer* mixer, Qt::WFlags f, ViewBase::ViewFlags vflags, GUIProfile *guiprof, KActionCollection *actionColletion)
@@ -76,6 +77,7 @@ ViewBase::ViewBase(QWidget* parent, const char* id, Mixer* mixer, Qt::WFlags f,
    action->setText(i18n("&Channels"));
    connect(action, SIGNAL(triggered(bool) ), SLOT(configureView()));
    connect ( _mixer, SIGNAL(controlChanged()), this, SLOT(refreshVolumeLevels()) );
+   connect ( _mixer, SIGNAL(controlsReconfigured(const QString&)), this, SLOT(controlsReconfigured(const QString&)) );
 }
 
 ViewBase::~ViewBase() {
@@ -101,7 +103,7 @@ QString ViewBase::id() const {
 
 bool ViewBase::isValid() const
 {
-   return (_mixSet->count() > 0 );
+   return ( _mixSet->count() > 0 || _mixer->dynamic() );
 }
 
 void ViewBase::setIcons (bool on) { KMixToolBox::setIcons (_mdws, on ); }
@@ -122,8 +124,8 @@ void ViewBase::createDeviceWidgets()
     {
         MixDevice *mixDevice;
         mixDevice = (*_mixSet)[i];
-	QWidget* mdw = add(mixDevice);
-	_mdws.append(mdw);
+        QWidget* mdw = add(mixDevice);
+        _mdws.append(mdw);
     }
     // allow view to "polish" itself
     constructionFinished();
@@ -203,16 +205,78 @@ void ViewBase::showContextMenu()
     _popMenu->popup( pos );
 }
 
+void ViewBase::controlsReconfigured( const QString& mixer_ID )
+{
+    if ( _mixer->id() == mixer_ID )
+    {
+        kDebug(67100) << "ViewBase::controlsReconfigured() " << mixer_ID << " is being redrawn (mixset contains: " << _mixSet->count() << ")";
+        setMixSet();
+        kDebug(67100) << "ViewBase::controlsReconfigured() " << mixer_ID << ": Recreating widgets (mixset contains: " << _mixSet->count() << ")";
+        createDeviceWidgets();
+
+        // We've done the low level stuff our selves but let elements
+        // above know what has happened so they can reload config etc.
+        emit redrawMixer(mixer_ID);
+    }
+}
 
 void ViewBase::refreshVolumeLevels()
 {
     // is virtual
 }
 
-Mixer* ViewBase::getMixer() {
+Mixer* ViewBase::getMixer()
+{
     return _mixer;
 }
 
+void ViewBase::setMixSet()
+{
+    if ( _mixer->dynamic()) {
+
+        // Check the guiprofile... if it is not the fallback GUIProfile, then
+        // make sure that we add a specific entry for any devices not present.
+        if ( 0 != _guiprof && MixerToolBox::instance()->fallbackProfile(_mixer) != _guiprof ) {
+            kDebug(67100) << "Dynamic mixer " << _mixer->id() << " is NOT using Fallback GUIProfile. Checking to see if new controls are present";
+
+            QList<QString> new_mix_devices;
+            MixSet ms = _mixer->getMixSet();
+            for (int i=0; i < ms.count(); ++i)
+                new_mix_devices.append("^" + ms[i]->id() + "$");
+            std::vector<ProfControl*>::const_iterator itEnd = _guiprof->_controls.end();
+            for ( std::vector<ProfControl*>::const_iterator it = _guiprof->_controls.begin(); it != itEnd; ++it)
+                new_mix_devices.removeAll((*it)->id);
+
+            if ( new_mix_devices.count() > 0 ) {
+                kDebug(67100) << "Found " << new_mix_devices.count() << " new controls. Adding to GUIProfile";
+                while ( new_mix_devices.count() > 0 ) {
+                    ProfControl* ctl = new ProfControl();
+                    ctl->id = new_mix_devices.takeAt(0);
+                    ctl->subcontrols = ".*";
+                    ctl->tab = _guiprof->_tabs[0]->name; // Use the first tab... not ideal but should work most of the time;
+                    ctl->show = "simple";
+                    _guiprof->_controls.push_back(ctl);
+                }
+                QString profileName;
+                profileName =  _mixer->id() + "." + id();
+                _guiprof->writeProfile(profileName);
+            }
+        }
+
+        // We need to delete the current MixDeviceWidgets so we can redraw them
+        while (!_mdws.isEmpty()) {
+            QWidget* mdw = _mdws.last();
+            _mdws.pop_back();
+            delete mdw;
+        }
+
+        // Clean up our _mixSet so we can reapply our GUIProfile
+        _mixSet->clear();
+    }
+    _setMixSet();
+}
+
+
 /**
  * Open the View configuration dialog. The user can select which channels he wants
  * to see and which not.
